#!/bin/bash
#
# Sign a neutral KEROS package with custom keys

# Directories
#############
TOOL_DIR=`dirname $0`
TOOL_DIR=`readlink -f ${TOOL_DIR}`
WORKSPACE="keros"
INITRD_GPG_KEYS_FILENAME="keros_gnupg.tar.gz"
ROOTFS_SQFSN_FILENAME="keros-neutral.sqfs"
GPG_USER=customer

. sign_lib.sh

INITRD_GPG_KEYS_FILE="${TOOL_DATA_DIR}/${INITRD_GPG_KEYS_FILENAME}"

### Other functions
##################

usage()
{
	echo -e "Usage: $0 <kerOS neutral.ipk>
\tKerOS neutral.ipk: KerOS IPK file without any signatures delivered by Kerlink"
}

if [ $UID != 0 ]; then
    echo "Please run this script with sudo or root privileges."
    exit 1
fi

if ! [ $# -eq 1 ]
then
	usage
	exit 1
fi


#######################
# MAIN PART
#######################

# Get original IPK file
IPK_KEROS_NEUTRAL_FILE="$1"

prepare_working_dir

#
# Extractions
ipk_extract ${IPK_KEROS_NEUTRAL_FILE} ${WORKSPACE}
itb_extract ${WORKING_DIR}/${WORKSPACE}/ipk/data/keros/${IPK_ITB_FILENAME} ${WORKSPACE}
initrd_extract ${WORKING_DIR}/${WORKSPACE}/itb/${ITB_INITRDGZ_FILENAME} ${WORKSPACE}

#
# Integration of custom GPG keys in initrd
initrd_install_gpg_keys ${INITRD_GPG_KEYS_FILE} ${WORKSPACE}

#
# Building ITB
initrd_build ${WORKSPACE} ${WORKING_DIR}/${WORKSPACE}/itb/${ITB_INITRDGZ_FILENAME}
itb_build ${WORKSPACE} ${WORKING_DIR}/${WORKSPACE}/ipk/data/keros/${IPK_ITB_FILENAME}

#
# Signatures
load_signinfo ${WORKING_DIR}/${WORKSPACE}/ipk/data/tmp/signinfo

itb_sign ${WORKING_DIR}/${WORKSPACE}/ipk/data/keros/${IPK_ITB_FILENAME} ${WORKING_DIR}/${WORKSPACE}/ipk/data/keros/${IPK_ITB_FILENAME}
tzsw_sign ${WORKING_DIR}/${WORKSPACE}/ipk/data/tmp/${TZSW_TZSW_FILENAME} ${WORKING_DIR}/${WORKSPACE}/ipk/data/tmp/${TZSW_TZSWS_FILENAME}
uboot_sign ${WORKING_DIR}/${WORKSPACE}/ipk/data/tmp/${UBOOT_UBOOT_FILENAME} ${WORKING_DIR}/${WORKSPACE}/ipk/data/tmp/${UBOOT_UBOOTS_FILENAME}
spl_sign ${WORKING_DIR}/${WORKSPACE}/ipk/data/tmp/${SPL_SPL_FILENAME} ${WORKING_DIR}/${WORKSPACE}/ipk/data/tmp/${SPL_SPLS_FILENAME}

# Keep same workspace so remove unsigned files
rm -f ${WORKING_DIR}/${WORKSPACE}/ipk/data/tmp/${TZSW_TZSW_FILENAME}
rm -f ${WORKING_DIR}/${WORKSPACE}/ipk/data/tmp/${UBOOT_UBOOT_FILENAME}
rm -f ${WORKING_DIR}/${WORKSPACE}/ipk/data/tmp/${SPL_SPL_FILENAME}

# Rename rootfs to be signed
mv ${WORKING_DIR}/${WORKSPACE}/ipk/data/keros/${ROOTFS_SQFSN_FILENAME} ${WORKING_DIR}/${WORKSPACE}/ipk/data/keros/${ROOTFS_SQFS_FILENAME}
gpg_sign ${WORKING_DIR}/${WORKSPACE}/ipk/data/keros/${ROOTFS_SQFS_FILENAME} ${WORKING_DIR}/${WORKSPACE}/ipk/data/keros/${ROOTFS_SQFS_FILENAME}.sig ${GPG_USER}

# Allow a change in IPK version (only for downgrade tests)
control_file="${WORKING_DIR}/${WORKSPACE}/ipk/control/${IPK_CONTROL_FILENAME}"
ipk_orig_version=$(grep Version ${control_file} | awk '{print $2}')

read -p "Please enter IPK version (unchanged to ${ipk_orig_version} if empty): " ipk_version
if [ "x${ipk_version}" != "x" ]
then
	final_version=${ipk_version}
else
	final_version=${ipk_orig_version}
fi
ipk_adapt_control ${WORKSPACE} ${final_version}

# IPK content is ready, generate the IPK itself
arch=$(grep Architecture ${control_file} | awk '{print $2}')
ipk_keross_file="keros-${final_version}-custom-signed_${arch}.ipk"
ipk_build ${WORKSPACE} ${TOOL_DIR}/${ipk_keross_file}

echo "Signed IPK is built: ${ipk_keross_file}"
