#!/bin/bash

SIGN_DIR=`dirname $0`
SIGN_DIR=`readlink -f ${SIGN_DIR}`/..
PR_TOOLS_DIR="${SIGN_DIR}/ProvenRun/tools"
export CST_PATH="${SIGN_DIR}/cst-2.3.2"

usage()
{
	echo -e "Usage: $0 <imx file> <HAB blocks>
\timx file:\t\tThe imx file to sign
\tHAB blocks:\tThe HAB blocks address (e.g. 0x00907400 0x00000000 0x0000bc00)"
}

if ! [ $# -eq 2 ]
then
	usage
	exit 1
fi


BINARY="$1"
HAB_BLOCKS="$2"

BINARY_DIR=`dirname $1`

rm -f ${BINARY_DIR}/${BINARY}-signed

cd ${PR_TOOLS_DIR}

# Select CSF template depending of type of PKI: Fast Authentication or not
if [ -f ${SIGN_DIR}/crts/CSF_usr_crt.pem ]
then
	cp -f ${SIGN_DIR}/data/imx_no_FA.cfg.tpl imx.cfg
else
	cp -f ${SIGN_DIR}/data/imx_FA.cfg.tpl imx.cfg
fi
sed -i '/Blocks/d' imx.cfg

# Test for mfgtool format
echo "${HAB_BLOCKS}" | grep -q ','
if [ $? -eq 0 ]
then
	HAB_BLOCK1="`echo ${HAB_BLOCKS} | cut -d ',' -f 1`"
	HAB_BLOCK2="`echo ${HAB_BLOCKS} | cut -d ',' -f 2`"
	echo -e "    Blocks = ${HAB_BLOCK1} \"${BINARY}\", ${HAB_BLOCK2} \"${BINARY}\"" >> imx.cfg
	# DCD address must be cleared for signature, as mfgtool will clear it.
	${PR_TOOLS_DIR}/mod_4_mfgtool.sh clear_dcd_addr ${BINARY}

	# generate the signatures, certificates, ... in the CSF binary
	${CST_PATH}/linux64/cst --o imx-csf.bin --i imx.cfg
	ret=$?

	# DCD address must be set for mfgtool to localize the DCD table.
	${PR_TOOLS_DIR}/mod_4_mfgtool.sh set_dcd_addr ${BINARY}
else
	echo -e "    Blocks = ${HAB_BLOCKS} \"${BINARY}\"" >> imx.cfg
	${CST_PATH}/linux64/cst --o imx-csf.bin --i imx.cfg
	ret=$?
fi

if [ $ret -eq 0 ]
then
	objcopy -I binary -O binary --pad-to 0x400 --gap-fill=0x00 imx-csf.bin imx-csf-padded.bin
	cat ${BINARY} imx-csf-padded.bin > ${BINARY}-signed

	echo "${BINARY}-signed is ready to use"

	# Cleanup
	rm -f imx-csf-padded.bin imx-csf.bin
else
	echo "Error during signature"
fi
