#!/bin/bash
#
# Secure the binaries of an IPK file by KerOS.

# Directories
#############
TOOL_DIR=`dirname $0`
TOOL_DIR=`readlink -f ${TOOL_DIR}`
WORKSPACE="secure_app"
GPG_USER=customer

. tools.sh

IPK_DIR="${WORKING_DIR}/${WORKSPACE}/ipk"
IPK_DATA_DIR="${IPK_DIR}/data"

# Files
#######
IPK_SEC_APP_FILENAME="secured_files.list"
IPK_SEC_APP_FILE="${IPK_DATA_DIR}/tmp/${IPK_SEC_APP_FILENAME}"

### Other functions
##################

usage()
{
	echo -e "Usage: $0 <application.ipk> <file 1>...<file n>
\tapplication.ipk: IPK file with application
\tfile X: A file to be checked by KerOS"
}

if [ $# -lt 2 ]
then
	usage
	exit 1
fi

#######################
# MAIN PART
#######################

prepare_working_dir

ORIG_IPK_FILE="$1"
echo "Original IPK file: ${ORIG_IPK_FILE}"

ipk_extract ${ORIG_IPK_FILE} ${WORKSPACE}

shift # To the list of files to sign

mkdir -p "${IPK_DATA_DIR}/tmp"
cd "${IPK_DATA_DIR}"
touch ${IPK_SEC_APP_FILE}
while [ $# -gt 0 ]
do
	file="$1"
	file=${file#"/"} # Remove first / if any
	echo "Add signature for file ${file}"

	# Only files present in IPK are accepted
	if ! [ -f ${file} ]
	then
		echo "Error: File does not exists in IPK!"
		exit 1
	fi

	# Check if any signature file is not already present
	if [ -f ${file}.sig ]
	then
		echo "Error: Signature file already exists in IPK!"
		exit 1
	fi

	# Generate signature
	gpg_sign ${IPK_DATA_DIR}/${file} ${IPK_DATA_DIR}/${file}.sig ${GPG_USER}

	# Append file in list of files to check by KerOS
	echo "/${file}" >> ${IPK_SEC_APP_FILE}

	echo "File ${file} signed"

	shift # to the next file to sign
done

# Generate signature of list of files file
gpg_sign ${IPK_SEC_APP_FILE} ${IPK_SEC_APP_FILE}.sig ${GPG_USER}

ipk_orig_filename=$(basename "${ORIG_IPK_FILE}")
ipk_secured_filename="${ipk_orig_filename%.*}-secured.${ipk_orig_filename##*.}"

# Build secured IPK file
ipk_build ${WORKSPACE} ${TOOL_DIR}/${ipk_secured_filename}

echo "Package IPK is secured: ${TOOL_DIR}/${ipk_secured_filename}"
